/*
The Golden Stool
Author: Dan (dan@dantom.com)
Version:  1.4  (29-June-2000)

Herein lies the adventure of the Golden Stool.  Those who read this text
will look upon the world with new eyes.  Those who seek to change the
runes shall become masters of their own fate...

Enter at your own risk!

For more information about the basic night soil system, take a look at
Dan.NightSoil in BYOND Hub.

This project also makes use of Dan.Maze to generate an "infinite" dungeon.

*/

//#define DEBUG

//The following hides debugging output when we compile without defining DEBUG
#ifdef DEBUG
#define debug world.log
#else
#define debug null
#endif


var
	NumPlayers
	Maze/maze
	savefile/SaveFile

world
	hub = "Dan.GoldenStool"
	turf = /turf/rock

	maxx = 30
	maxy = 30
	maxz = 2   //so maze remembers only the previous level


world/New()
	var/Level
	SaveFile = new("GoldenStool.sav")

	//we use the total number of players to guage the "creature density"
	SaveFile.cd = "/players"
	NumPlayers = SaveFile.dir.len

	//recall how deep we were last time
	SaveFile["/level"] >> Level
	if(Level <= 0) Level = 1

	//initialize the maze, which manages our "infinite" dungeon
	maze = new()
	maze.TunnelType = /turf/tunnel
	maze.RoomType = /turf/floor
	maze.WallType = /turf/rock
	maze.Initialize(locate(2,2,1),locate(world.maxx-2,world.maxy-2,2),Level)


mob/Login()
	//check for save file
	if(!usr.loc)
		SaveFile.cd = "/players"
		if(src.ckey in SaveFile.dir)  //old player
			SaveFile.cd = src.ckey
			if("died" in SaveFile.dir) usr.GotoAfterlife()
			else
				src.Read(SaveFile)
				usr << "You awake from your meditations."

		else  //new player
			new/obj/Scroll/introduction(usr)
			new/obj/Food/fortune_cookie(usr)
			NumPlayers++

			usr << "Welcome to Night Soil, the Adventure of the Golden Stool."

	if(NumPlayers != 1)
		var/mob/M
		var/count = 0
		for(M) if(M.client) count++
		world << "There are [NumPlayers] adventurers ([count] are exploring now)."

	//materialize on an alter
	if(!usr.loc) UsrGotozLevel(maze.Level,/turf/altar)


//meditation allows the player to gracefully log out
mob/verb/meditate()
	var/old_loc = usr.loc
	if(usr.enemy)
		usr << "You are not nearly calm enough to concentrate."
		return

	view() << "[usr] begins to meditate."
	sleep(50)
	if(usr.loc != old_loc) return
	usr << "The voice of your master sounds in the distance..."
	sleep(50)
	if(usr.loc != old_loc) return
	usr << "Your soul floats free at last!"
	oview() << "[usr] fades from sight."

	SaveFile.cd = "/players"
	SaveFile.dir.Remove(usr.ckey) //clear death flag, etc.
	SaveFile.cd = usr.ckey
	usr.Write(SaveFile)

	SaveFile["/level"] << maze.Level

	del(usr)

//this is called by Night Soil when the player dies
mob/GotoAfterlife()
	if(usr.loc)  //don't save when player is logging in
		//otherwise, force a save at this point
		SaveFile.cd = "/players"
		SaveFile.cd = src.ckey
		SaveFile["died"] << 1
		src.Write(SaveFile)
	..()


//this is called when the maze needs to create a new level
Maze/GenLevel(Loc0,Loc1,Level)
	..()  //make the default maze

	//add a few extra things to the map (stairs and alter)
	ReplaceATurfInBlock(/turf/tunnel, /turf/downstairs, Loc0, Loc1)
	ReplaceATurfInBlock(/turf/tunnel, /turf/altar, Loc0, Loc1)
	if(Level>1)  //No upstairs on first level
		ReplaceATurfInBlock(/turf/tunnel, /turf/upstairs, Loc0, Loc1)

	//add the creatures and items
	PopulateBlock(Loc0,Loc1,Level)


proc/PopulateBlock(turf/Loc0,turf/Loc1,Level)
	var
		Nmob = NumPlayers
		Nobj = NumPlayers
		Type
		turf/T
		mob/M


	if(!Nmob) Nmob = 1
	if(!Nobj) Nobj = 1
	Nmob *= round(0.01 * (Loc1.x-Loc0.x+1)*(Loc1.y-Loc0.y+1)*(Loc1.z-Loc0.z+1))
	Nobj *= round(0.01 * (Loc1.x-Loc0.x+1)*(Loc1.y-Loc0.y+1)*(Loc1.z-Loc0.z+1))

	//populate mobs
	debug << "creating [Nmob] mobs"
	while(Nmob--)
		Type = world.MobTable(Level)
		T = LocateATurfInBlock(/turf/floor,Loc0,Loc1)
		M = new Type(T)

		//create mob's treasure
		T=rand(M.rand_items)
		while(T-- && Nobj--)
			Type = world.ObjTable(Level)
			new Type(M)

	//populate objs
	debug << "creating [Nobj] objs"
	while(Nobj-- > 0)
		Type = world.ObjTable(Level)
		T = LocateATurfInBlock(/turf/floor,Loc0,Loc1)
		new Type(T)



turf/altar
	text = "**ooo"
	density = 0

	verb
		kneel()
			usr.step_mode = SIT_STEP
			view() << "[usr] kneels before [src]."
			sleep(200 + usr.karma)
			if(usr && usr.loc == src)
				usr << "The gift of prescience has been granted to you!"
				new/obj/Food/fortune_cookie(usr.loc)


		set_name(T as text)
			set name = "my name"
			if(usr.karma > 0) usr << "You must clear the sin of your old name first."
			else usr.SetName(T)

		set_description(T as text)
			set name = "my description"
			if(usr.karma > 0) usr << "You must clear your tarnished image first."
			else usr.desc = T

		set_gender(G in list("female","male","neuter"))
			set name = "my gender"
			if(usr.karma > 0)
				usr << "You must clear your tarnished image first."
				return

			usr.gender = G

	Enter(O)
		if(istype(O,/obj/Food) && usr)
			//user is offering food to the alter (good)
			usr.AddKarma(-O:karma)
		return ..()

	Exit(O)
		if(istype(O,/obj/Food) && usr)
			//user is taking away food from the alter (bad!)
			usr.AddKarma(2*O:karma)
		return ..()


client
	Center() //the center key picks stuff up or uses stairs
		var/obj/O
		var/i
		var/turf/T = usr.loc

		for(i=T.contents.len,i,i--)
			O = T.contents[i]
			if(!isobj(O)) continue

			O.get() //pick up top object
			return

		T.Use()


var/const/FEET_PER_LEVEL = 20
mob/Stat()
	//show how deep we are
	stat("depth",maze.GetLevel(usr.z) * FEET_PER_LEVEL)
	return ..() //Night Soil fills in our other stats


//Move player 20 feet up.
world/UsrUpLevel()
	var/l = maze.GetLevel(usr.z) - 1
	var/z
	if(l > 0) z = maze.GetzLevel(l)

	if(z)
		UsrGotozLevel(z,/turf/downstairs)
		return 1

	else usr << "Your passage is frozen solid."


//Move player 20 feet down.
world/UsrDownLevel()
	var/l = maze.GetLevel(usr.z) + 1
	var/z = maze.GetzLevel(l)

	if(z)
		UsrGotozLevel(z,/turf/upstairs)
		return 1

	else usr << "Your passage is frozen solid."


//Move player to an existing z-level on the map.
proc/UsrGotozLevel(Z,Type)
	var/Loc = LocateTurfInBlock(Type,locate(1,1,Z),locate(world.maxx,world.maxy,Z))
	var/turf {L0; L1}
	L0 = locate(1,1,Z)
	L1 = locate(world.maxx,world.maxy,Z)
	world.status = "[Z*FEET_PER_LEVEL] feet down"
	if(!Loc) debug << "Failed to find [] in ([],[],[]) to ([],[],[])."(Type,L0.x,L0.y,L0.z,L1.x,L1.y,L1.z)
	if(!Loc || !usr.MoveNoBump(Loc))
		usr.MoveNoBump(LocateTurfInBlock(/turf/tunnel,locate(1,1,Z),locate(world.maxx,world.maxy,Z)))


world
	ObjTable(Level)
		//The Golden Stool resides on every level starting from 30 on down.
		if(Level >= 30 && !locate(/obj/golden_stool)) return /obj/golden_stool
		return ..()


//Whooohooo!  Here it is.  You found it!!!
obj/golden_stool
	name = "\improper Golden Stool"
	desc = "The Golden Stool of the Lizard Queen!"
	text = "GoldenStool"

	Move(Trg)
		if(ismob(Trg) && src.loc) Trg << "[src] is much too heavy!"
		else return ..()

	verb
		sit()
			set src in oview(0)
			var/count
			var/old_loc = usr.loc

			view() << "[usr] sits on [src]."
			for()
				sleep(100)
				if(usr.loc != old_loc) return
				switch(count++)
					if(0)
						oview() << "[usr] falls into a trance!"
						usr << "You have a vision!"

					if(1)
						usr << "\nIt is night in the pasture.\
						        You stand reeling beneath the stars."

					if(2)
						usr << "\nSuddenly an enormous dark shape looms above the horizon.  \
						        Her sibilant voice cuts through you like an ice wind.  \
						        It is the Lizard Queen!"

					if(3)
						usr << "\n\"So you have come at last, mortal.\"\
						        \n\"What dreams I have had.\"\
						        \n\"And they all begin with you!\"\
						        \nShe draws closer, towering over you."

					if(4)
						usr << "\n\"You are the chosen one.\"\
						        \n\"The messenger of my awakening.\"\
						        \nHer wings spread wide and she leaps into the air.\
						        \n\"Prepare my way and your reward shall be great.\""

					if(5)
						usr << "\nHer shape recedes into the night.\
						        \nA final hiss is all that remains.\
						        \n\"Do not oppose me, [usr].\"\
						        \n\"I have not eaten for millenia...\""

					else break

			//suck away all karma
			usr.AddKarma(-usr.karma)

/obj/Scroll/introduction
	read()
		usr << "\"Computer games?!\"  Your master glares at you.  \
					\"You think it would have been fun to play \
					computer games when the fate of the world was \
					at stake?\"  He lapses into a moody silence."
		sleep(100)

		usr << "\n\"You have learned <em>nothing</em> from your study of \
					the Age of Machines!\"  He silences your protest \
					with a wave of his hand.  \"It is my fault.  \
					You are like your father--no patience for history.  \
					You want action.\""
		sleep(100)

		usr << "\n\"So I will send you on a quest into the very \
					depths of history.  The age you will study \
					makes the Age of Machines appear like yesterday. \
					And its fall was no less cataclysmic.  He closes \
					his eyes and hums for a while."
		sleep(100)

		usr << "\n\"Deep in the bowls of the earth, in a tomb of dust \
					and fallen pride, lies the dark and hollow fortress \
					of the final Lizard Queen.  Where she fled with \
					all her plunder and when she died no books will \
					say.  That dark age left no histories but ash \
					and bone beneath the clay."
		sleep(100)

		usr << "\nYour flesh crawls as you listen to the story of \
					giant lizards.  Your master's words hold you as \
					though in a trance.  Is it the serpentine glint \
					in his eye?  Or is it the sibilant hiss of his \
					voice?  You shiver."
		sleep(100)

		usr << "\nAt last, the story complete, he produces a pouch \
					from his robes and empties the contents into your \
					hand.  \"I have taught you that the future is \
					unwritten.  But there are clues buried in unlikely \
					places.  Take this.  It will help you.\"  You tuck \
					what looks like a fortune cookie into your belt. \
					Whatever."
		sleep(100)

		usr << "\n\"Go now, impatient one.  I have nothing more to \
					teach you until this quest is finished.  Find \
					the Lizard Queen's lost throne room.  It is in \
					the deepest recesses of the Night Caves.  Perhaps \
					those dark wastelands will impress upon you the true \
					nature of cataclysm."
		sleep(100)

		usr << "\n\"One more thing.  There are legends.  She had \
					various instruments--dangerous ones.  Take \
					nothing from that place but the knowledge you \
					acquire.  And if you find it, stay away from \
					Her Golden Stool..."
		sleep(100)

		usr << "\nAnd with this advice firmly in mind you find yourself \
              here in the gloom on a quest for the Golden Stool of the \
	           Lizard Queen."
